program cpdf

  implicit none
  
  character*80 :: input = 'jfm_incr10.dat'
  character*80 :: output = 'jpdf_incr10_5.dat'
  character*80 :: output2 = 'cpdf_incr10_5.dat'
  integer,parameter :: tau = 5
  
  ! Korrekturfaktor zur Daumenregel-Bandbreitenmatrix
  real*8 :: bwfac = 0.8d0
  integer :: iker = 1
  ! iker = 1 : Epanechnikov
  ! iker = 2 : Gauss
  
  integer,dimension(2) :: nsmp = (/ 500 , 500 /)
  real*8,allocatable :: jpdf(:,:), axis1(:), axis2(:), x(:,:)
  real*8,dimension(2) :: mini,maxi,dx,mean, x0, diff
  real*8,dimension(2,2) :: cov, h_mat,h_inv
  integer,dimension(2) :: ind, iub, ilb, ir
  
  integer :: ndp,ios,i,j,k
  real*8 :: d1,d2, det_h,norm, gauss, epa
  
  real*8,allocatable :: pdf1(:), pdf2(:), cpdf12(:,:), cpdf21(:,:)
  
  allocate(jpdf(nsmp(1),nsmp(2)))
  allocate(axis1(nsmp(1)))
  allocate(axis2(nsmp(2)))
  
  allocate( pdf1(nsmp(1)) , pdf2(nsmp(2)) )
  allocate( cpdf12(nsmp(1),nsmp(2)) , cpdf21(nsmp(2), nsmp(1)) )
  
  print*, ' '
  print*, 'Lese Daten von Datei ', input,  '...'
  
  open(10,file=input,status='old')
  
  ndp = -1
  ios = 0
  
  do while (ios==0)
    read(10,*,iostat=ios) x0(1)
    ndp=ndp+1 
  end do
  
  rewind(10)
  
  print*, ndp, 'Datenpunkte'
    
  allocate(x(2,ndp))
  
  do i=1,ndp
    read(10,*) x(1,i)
  end do
  
  do i=1,ndp-tau
    x(2,i) = x(1,i+tau)
  end do
  
  close(10)
  print*, 'Fertig.'
  
  mini = x(:,1)
  maxi = x(:,1)
  
  do i=2,ndp-tau
    do j=1,2
      if ( x(j,i)<mini(j) ) mini(j) = x(j,i)
      if ( x(j,i)>maxi(j) ) maxi(j) = x(j,i)
    end do
  end do
  
  mini = mini - (maxi-mini)*0.1d0
  maxi = maxi + (maxi-mini)*0.1d0

  print*, ''
  print*, 'Minimum:', mini
  print*, 'Maximum:', maxi
  print*, ''
  print*, 'Bestimme Koordinatenachse ...'
  
  dx = (maxi - mini)/real(nsmp-1,8)
  
  do i=1,nsmp(1)
    axis1(i) = mini(1) + real(i-1,8)*dx(1)
  end do

  do i=1,nsmp(2)
    axis2(i) = mini(2) + real(i-1,8)*dx(2)
  end do
  print*, 'Fertig.'
  
  print*, ''
  print*, 'Bestimme Bandbreitenmatrix ...'
  
  cov = 0.0d0
  mean = 0.0d0
  do i=1,ndp-tau
    mean = mean + x(:,i)
  end do
  mean = mean/real(ndp-tau,8)
  print*, 'Mittelwerte:', mean
  
  do i=1,ndp-tau
    d1 = x(1,i) - mean(1)
    d2 = x(2,i) - mean(2)
    cov(1,1) = cov(1,1) + d1*d1
    cov(2,1) = cov(2,1) + d1*d2
    cov(2,2) = cov(2,2) + d2*d2
  end do
  cov(1,2) = cov(2,1)
  cov = cov/real(ndp-tau,8)
  
  call sqrt2x2mat(cov(1,1),cov(2,1),cov(2,2),h_mat)
   
  h_mat = bwfac*h_mat*real(ndp,8)**(-1.0d0/6.0d0)
  print*, 'H:', h_mat
  
  ! Falls Epanechnikov-Kern, bestimme Reichweite der Kernels
  if (iker == 1) then
    ! Berechne Spur als obere Schranke des groessten Eigenwertes
    d1 = h_mat(1,1) + h_mat(2,2)
    ! ir = obere Schranke der Reichweite des Kernels als Anzahl der Gitterpunkte
    ir = int( d1*sqrt(6.0d0)/dx ) + 2
    print*, 'Range of Epanechnikov-Kernel:', ir
  end if
  
  det_h = h_mat(1,1)*h_mat(2,2) - h_mat(2,1)*h_mat(1,2)
  
  call inverse2x2mat(h_mat,h_inv)
    
  print*, ''
  print*, 'Berechne jpdf ...'
  
  jpdf = 0.0d0
  
  if (iker == 2) then ! Gauss-Kern
    do i=1,nsmp(1)
    
      if (mod(i,nsmp(1)/10)==0) then
	print*, i, 'von', nsmp(1)
      end if
      x0(1) = axis1(i)
      
      do j=1,nsmp(2)
      
	x0(2) = axis2(j)
	
	do k=1,ndp-tau

	  diff = x(:,k) - x0
	  diff = matmul(h_inv,diff)
	  jpdf(i,j) = jpdf(i,j) + gauss(diff)

	end do
      
      end do
      
    end do
    
    jpdf = jpdf/(real(ndp-tau,8)*det_h)
    jpdf = jpdf/(2.0d0*acos(-1.0d0))
    
  else if (iker == 1) then ! Epanechnikov
  
    do i=1,ndp-tau
  
      if (mod(i,ndp/10)==0) then
	print*, i, 'von', ndp
      end if
  
      ! Bestimme Intervall der Indizes der Gitterpunkte, an denen der Kernel ungleich null sein kann
      ind = int( (x(:,i) - mini)/dx ) + 1
      ilb = ind - ir
      iub = ind + ir
      
      do j=1,2
	if (ilb(j) < 1) ilb(j) = 1
	if (iub(j) > nsmp(j)) iub(j) = nsmp(j)
      end do
      
      ! Schleife ueber alle GP, an dene Kernel ungleich null sein kann
      do j=ilb(1), iub(1)
      
	x0(1) = axis1(j)
	
	do k=ilb(2), iub(2)
	
	  x0(2) = axis2(k)
	  diff = x(:,i) - x0
	  diff = matmul(h_inv,diff)
	  jpdf(j,k) = jpdf(j,k) + epa(diff)
	
	end do
	
      end do
      
    end do
    
    jpdf = jpdf/(real(ndp-tau,8)*det_h)
    jpdf = jpdf/(18.0d0*(acos(-1.0d0)))
    
  end if
  
  open(12,file=output,status='replace')
  norm = .0d0
  do i=1,nsmp(1)
    do j=1,nsmp(2)
      write(12,*) axis1(i) , axis2(j) , jpdf(i,j)
      norm = norm + jpdf(i,j)
    end do
    write(12,*) ''
  end do
  close(12)
  norm = norm*dx(1)*dx(2)
  
  print*, 'Norm:', norm
  
    
  print*, ''
  print*, 'Berechne Martingale ...'
  
  pdf1 = 0.0d0
  do i=1,nsmp(1)
    do j=1,nsmp(2)
      pdf1(i) = pdf1(i) + jpdf(i,j)
    end do
  end do
  pdf1 = pdf1*dx(2)
  
  pdf2 = 0.0d0
  do i=1,nsmp(2)
    do j=1,nsmp(1)
      pdf2(i) = pdf2(i) + jpdf(j,i)
    end do
  end do
  pdf2 = pdf2*dx(1)
  
  norm = 0.0d0
  open(13,file='martingal1.dat',status='replace')
  do i=1,nsmp(1)
    write(13,*) axis1(i) , pdf1(i)
    norm = norm + pdf1(i)
  end do
  close(13)
  norm = norm*dx(1)
  print*, 'Norm:', norm
  
  norm = 0.0d0
  open(13,file='martingal2.dat',status='replace')
  do i=1,nsmp(2)
    write(13,*) axis2(i) , pdf2(i)
    norm = norm + pdf2(i)
  end do
  close(13)
  norm = norm*dx(2)
  print*, 'Norm:', norm
  
  print*, 'Fertig.'
  print*, ''
  print*, 'Berechne CPDFs ...'
  
  cpdf12 = 0.0d0
  cpdf21 = 0.0d0
  
  do i=1,nsmp(1)
    do j=1,nsmp(2)
      if (pdf2(j)>0.0d0) then
	cpdf12(i,j) = jpdf(i,j) / pdf2(j)
      end if
      if (pdf1(i)>0.0d0) then
	cpdf21(j,i) = jpdf(i,j) / pdf1(i)
      end if
    end do
  end do
  
  open(12,file=output2,status='replace')
  
  do i=1,nsmp(1)
    do j=1,nsmp(2)
      write(12,"(4(e13.6,x))") axis1(i) , axis2(j) , cpdf12(i,j) , cpdf21(j,i)
  
    end do
    write(12,*) ''
  end do
  close(12)
    
end program

!====================================================================

real*8 function gauss(x)

  implicit none
  
  real*8,dimension(2) :: x
  
! Normierungskonstante wird zum Schluss eingefuegt, um Rechenzeit zu sparen
  gauss = exp(-0.5d0*(x(1)*x(1)+x(2)*x(2)))

end function

!====================================================================

real*8 function epa(x)

  implicit none
  
  real*8,dimension(2) :: x
  real*8 r2
  
  r2 = x(1)*x(1) + x(2)*x(2)
  
  if (r2.lt.6.0d0) then
    epa = 6.0d0 - r2
  else
    epa = 0.0d0
  end if
! Normierungskonstante wird zum Schluss eingefuegt, um Rechenzeit zu sparen
  
end function





